﻿using Dynamicweb.DataIntegration.EndpointManagement;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.LiveIntegration.EndpointMonitoring;
using System;
using System.Collections.Generic;

namespace Dynamicweb.Ecommerce.LiveIntegration.Connectors
{
    /// <summary>
    /// Endpoint Connector
    /// </summary>
    internal class EndpointConnector : ConnectorBase
    {
        internal override EndpointInfo GetEndpoint()
        {
            return new EndpointInfo(GetEndpoint(Settings.Instance.Endpoint));
        }

        internal Endpoint GetEndpoint(string multipleUrlsText)
        {
            if (!string.IsNullOrEmpty(multipleUrlsText))
            {
                var endpoint = UrlHandler.Instance.GetEndpoint(multipleUrlsText, false);
                if (endpoint == null)
                {
                    throw new ArgumentException("Cannot find appropriate endpoint. Check Endpoint settings.");
                }
                return endpoint;
            }
            else
            {
                return null;
            }
        }

        internal EndpointInfo GetEndpointInfo(Endpoint endpoint)
        {
            return new EndpointInfo(endpoint);
        }

        /// <summary>
        /// Determines whether the web service connection is available.
        /// </summary>
        internal override bool IsWebServiceConnectionAvailable(EndpointInfo endpoint)
        {
            if (!Settings.Instance.IsLiveIntegrationEnabled)
            {
                return false;
            }
            if (endpoint?.Endpoint == null)
            {
                endpoint = new EndpointInfo(GetEndpoint(Settings.Instance.Endpoint));
            }
            Exception error = null;
            return ExecuteConnectionAvailableRequest(endpoint, out error);
        }

        internal bool ExecuteConnectionAvailableRequest(Endpoint endpoint, out Exception error)
        {
            return ExecuteConnectionAvailableRequest(GetEndpointInfo(endpoint), out error);
        }

        internal override string Execute(EndpointInfo endpoint, string request, TimeSpan responseTimeout)
        {
            string response = null;
            if (endpoint?.Endpoint != null)
            {
                EndpointService endpointService = new EndpointService();
                response = endpointService.Execute(endpoint.Endpoint.Id, endpointService.GetDynamicwebServiceSoapBody(request));
                EndpointMonitoringService.Success(endpoint);
            }
            return response;
        }

        internal override string Execute(string request)
        {
            return Execute(GetEndpointInfo(UrlHandler.Instance.GetEndpoint()), request);
        }

        internal override IEnumerable<string> GetUrls(string multipleUrlsText)
        {
            return UrlHandler.Instance.GetEndpointUrls(multipleUrlsText);
        }

        internal override bool IsConnectionAvailableFromBackend(string endpointId, out string error)
        {
            error = null;
            var endpoint = UrlHandler.Instance.GetEndpoint(endpointId, false);
            if (endpoint != null)
            {
                Exception ex = null;
                if (ExecuteConnectionAvailableRequest(endpoint, out ex))
                {
                    return true;
                }
                else
                {
                    error = $"Can not connect to the endpoint: {endpoint.Name} error: {ex?.Message}.";
                    return false;
                }
            }
            else
            {
                error = $"Endpoint with Id:{endpointId} can not be found.";
                return false;
            }
        }
    }
}
